#ifndef __FECONSUMER_OPTIONS_H__
#define __FECONSUMER_OPTIONS_H__

#include <iostream>
#include "FdsfeApi.h"

namespace FactSet {
namespace Datafeed {

struct FEConsumerOptionsImpl;

class FDSFE_API FEConsumerOptions {
  public:
  
    enum Mode {
      SERVER, // live data from server
      REPLAY  // recorded data from file.
    };

    FEConsumerOptions();
    FEConsumerOptions(const FEConsumerOptions& other);
    FEConsumerOptions& operator=(const FEConsumerOptions& other);
    ~FEConsumerOptions();

    FEConsumerOptions& set_worker_count(int value);
    FEConsumerOptions& set_enable_compression(bool value);
    FEConsumerOptions& set_high_watermark(int value);
    FEConsumerOptions& set_data_channel_heartbeat_interval(int value);
    FEConsumerOptions& set_data_channel_heartbeat_timeout(int value);
    FEConsumerOptions& set_data_channel_heartbeat_ttl(int value);
    FEConsumerOptions& set_mode(Mode mode, std::string host="");

    bool get_worker_count(int& value_out) const;
    bool get_enable_compression(bool& value_out) const;
    bool get_high_watermark(int& value_out) const;
    bool get_data_channel_heartbeat_interval(int& value_out) const;
    bool get_data_channel_heartbeat_timeout(int& value_out) const;
    bool get_data_channel_heartbeat_ttl(int& value_out) const;
    bool get_mode(Mode& value_out) const;
    bool get_replay_host(std::string& value_out) const;
  private:
    friend struct FEConsumerOptionsImpl;
    FEConsumerOptionsImpl* m_impl;
};

FDSFE_API std::ostream& operator<<(std::ostream& os, const FEConsumerOptions& opt);

}  // namespace Datafeed
}  // namespace FactSet

#endif