#ifndef __MD_LIB_MESSAGE_PRINTER_H__
#define __MD_LIB_MESSAGE_PRINTER_H__

#include <MD/private/prefix.h>
#include <MD/MD_FieldDefs.h>
#include <iostream>

namespace MD {
  class MD_Message;
  class MD_Variant;
  class MD_FieldMap;
}

// Usage:
//
// If msg is a MD::MD_Message, then
// these statements all print the same thing:
//
// cout << msg << endl;
// cout << MD_MessagePrinter(msg) << endl;
// cout << MD_MessagePrinter(msg, NORMAL) << endl;
// cout << MD_MessagePrinter(msg, NORMAL, true) << endl;
//
// Pass false as the third argument to print fids instead of field names.
// cout << MD_MessagePrinter(msg, NORMAL, false) << endl;
//
// Say you have a message with the price EXCH_LAST stored in the message as
// three fields, EXCH_LAST_WHOLE, EXCH_LAST_NUM, EXCH_LAST_DENOM.
// If you want to have these three fields printed as a float with the field
// name EXCH_LAST, use:
//
// cout << MD_MessagePrinter(msg, PRICE) << endl;
//

namespace MD {

class MD_MsgPrinter
{
public:
  typedef enum {
    NORMAL,
    PRICE
  } print_style_t;

  const MD::MD_Message    & m_msg;
  print_style_t                         m_style;
  bool                                  m_resolve_fids_to_field_names;

  MD_MsgPrinter(const MD::MD_Message & msg, print_style_t style = NORMAL, bool resolve = true)
    : m_msg(msg), m_style(style), m_resolve_fids_to_field_names(resolve)
  {}

  void print(std::ostream & os = std::cout) const;

private:
  void printFieldNormal(MD::FIELDS::FieldID fid,
                        const MD::MD_Variant & field_value,
                         std::ostream & os) const;

  void printFieldPrice(MD::FIELDS::FieldID fid,
                       const MD::MD_Variant & field_value,
                       std::ostream & os) const;

  void print_field_label(MD::FIELDS::FieldID fid, std::ostream & os) const;
};

std::ostream & operator << (std::ostream & os, const MD_MsgPrinter & printer);

}//MD

#endif // __MD_LIB_MESSAGE_PRINTER_H__

