#ifndef __MD_LIB_PUBLISHER_H__
#define __MD_LIB_PUBLISHER_H__

#include <string>

#include <fire/detail/RT_Publisher.h>
#include <MD/MD_OwnershipQueue.h>
#include <MD/MD_Message.h>
#include <fire/PostOffice.h>
#include <fire/ConfigFile.h>
#include <fire/Selector.h>
#include <fire/Selector2.h>
#include <fire/rt_api.h>
#include <fire/sys/time.h>

namespace FIRE_NAMESPACE {
template <typename T>
class MD_OwnershipQueue;
}

namespace MD {

void rt_to_md(MD::MD_Message* dest, const fire::RT_Message &src);

//template<class S = fire::Selector> class MD_Publisher_;
template<typename S = fire::Selector> class MD_Publisher_ : public FIRE_NAMESPACE::RT_Publisher {
public:
  MD_Publisher_(const std::string & app_name, const FIRE_NAMESPACE::ConfigFile & cfg);
  MD_Publisher_(const std::string & app_name, const std::string & key_appendage);
  virtual ~MD_Publisher_();

  virtual int on_request(MDS::OwnershipQueue<MD::MD_Message> & queue, const std::string & symbol, const std::string & iso) = 0;

  int send_update(const MD::MD_Message & message);
  void print_stats(std::ostream &os, FIRE_NAMESPACE::sys_time_t ctime, int seconds_delta);
  virtual void publish_state_change(const bool){}

private:
  // RT_Publisher callback
  virtual int handle_request(const FIRE_NAMESPACE::RT_Request &req, bool* complete);
  using fire::RT_Publisher::send_update;
  void send_update(const FIRE_NAMESPACE::RT_Message &rsp);
  int send_reply(const FIRE_NAMESPACE::RT_Request &req, FIRE_NAMESPACE::RT_Message &rsp, bool* complete);

  // non-virtual callbacks for internal purposes
  void on_request_dispatch(FIRE_NAMESPACE::sys_time_t ctime, void* closure);
  void on_flush_publisher (FIRE_NAMESPACE::sys_time_t ctime, void* closure);
  void on_dump_wl(const std::string &from, const std::string &subj, const std::string &body);
  void on_ready_for_requests(const std::string &from, const std::string &subj, const std::string &body);

  // connects to mr
  void rt_connect(FIRE_NAMESPACE::sys_time_t, int, void*);
  // tells mr that we are ready to handle requests
  void set_listening_mode(bool start_listening=true);
  // disconnects from mr and tries to reconnect every so often
  void disconnect_publisher();
  void constructor_helper();
  // fetch secondary key from service config
  void get_secondary_key(const fire::ConfigFile& cfg);

  // state
  std::string my_name;
  std::string my_service;
  std::string rt_connection_string;
  FIRE_NAMESPACE::RT_FieldMap* fieldmap;
  typedef typename S::template Socket<MD_Publisher_> SocketType;
  SocketType notify_sock;
  typedef typename S::template Timer<MD_Publisher_> TimerType;
  TimerType rt_retry_timer;
  FIRE_NAMESPACE::PostOffice::MailBox dump_wl;
  FIRE_NAMESPACE::PostOffice::MailBox m_ready_for_requests;
  int rt_retry_timer_val;
  std::string m_key_appendage;
  bool m_close_on_snapshot_failure;
  bool m_start_listening_on_connect;
  bool m_old_style_publisher;
  MD::FIELDS::FieldID m_secondary_key;
  bool m_secondary_key_not_required;

  struct Statistics
  {
    Statistics() { clear(); }

    void clear()
    {
      in_count = 0;
      in_size = 0;
      out_count = 0;
      out_size = 0;
      out_fids = 0;
    }

    unsigned int in_count;
    unsigned int in_size;
    unsigned int out_count;
    unsigned int out_size;
    unsigned int out_fids;
  };
  Statistics m_stats;
};

// We define the original type MD_Publisher here so we dont need to change any other code
typedef MD_Publisher_<> MD_Publisher; // uses Selector (v1) as before
typedef MD_Publisher_<fire::Selector2> MD_Publisher_Selector2; // uses Selector2

} // FIRE_NAMESPACE

#endif // __MD_LIB_PUBLISHER_H__
