#ifndef __MD_LIB_TYPEMAP_H__
#define __MD_LIB_TYPEMAP_H__

#include <map>
#include <string>
#include <vector>
#include <mutex>

#include <MD/MD_sys_types.h>
#include <MD/MD_Types.h>

  namespace MD {
    namespace DATATYPES {

      struct type_rec {
        MD::DATATYPES::TypeID id;
        std::string name;
        uint8_t size;
        
        bool operator< (const type_rec & rhs) const {
          return this->name < rhs.name;
        }
      };

    }


  class MD_TypeMap {
  public:
    typedef std::map <std::string, MD::DATATYPES::type_rec> data_type;
    typedef std::vector <data_type::const_iterator> id_index_type;
    typedef data_type::const_iterator const_iterator;
    
    static const MD_TypeMap & instance();
    static void destroy();
    
    // Search for type_rec structures in the type map by id or
    // name.  If the type_rec is found, they return a const_iterator
    // to that type_rec.  If the type_rec is not found, they return
    // an end() iterator.
    MD_TypeMap::const_iterator findById (MD::DATATYPES::TypeID id) const;
    MD_TypeMap::const_iterator findByName (const std::string & name) const;
    
    // const_iterator access to MD_TypeMap
    MD_TypeMap::const_iterator begin() const { return m_types.begin(); }
    MD_TypeMap::const_iterator end()   const { return m_types.end();   }
    
    size_t size() const { return m_types.size(); }
    
  private:
    friend class MD_FieldMap;
    MD_TypeMap();
    ~MD_TypeMap() {}
    
    data_type m_types;
    id_index_type m_id_index;
    MD::DATATYPES::TypeID m_max_id;
    static std::mutex m_lock;
  };
  
  
  // Implementation of inline methods goes here, to keep the class
  // declaration clean.
  
  inline MD_TypeMap::const_iterator
    MD_TypeMap::findById (MD::DATATYPES::TypeID id) const {
    if (id <= m_max_id) {
      return m_id_index[id];
    } else {
      return end();
    }
  }
  
  inline MD_TypeMap::const_iterator
    MD_TypeMap::findByName (const std::string & name) const {
    return m_types.find(name);
  }

}

#endif // __MD_LIB_TYPEMAP_H__
