// *******************************************************************//
// ** File: Types.h
// ** Description:
// **      File which contains all of the datatypes that we currently 
// **      send over the wire.
// **
// *******************************************************************//

#ifndef __MD_LIB_TYPES_H__
#define __MD_LIB_TYPES_H__

#include <MD/private/prefix.h>

#ifndef MD_LIB_WIN
#include <inttypes.h>
namespace MD { namespace DATATYPES { typedef uint8_t TypeID; } }
#else
namespace MD { namespace DATATYPES { typedef unsigned char TypeID; } }
#endif

namespace MD {
  namespace DATATYPES {
#ifdef MD_LIB_WIN
    const TypeID DUMMY         = TypeID(-1);
    const TypeID OPTI_FLOAT    = 15; // @@ Temporary
#endif
    
    const TypeID RAW           = 0;
    const TypeID CHAR          = 1;
    const TypeID INT           = 2;
    const TypeID SHORT         = 3;
    const TypeID FLOAT         = 4;
    const TypeID DOUBLE        = 5; 
    const TypeID STRING        = 6;
    const TypeID VECTOR        = 7;
    const TypeID MSG_T         = 8;
    const TypeID DATE          = 9;
    const TypeID TIME          = 10;
    const TypeID FRACTION      = 11;
    const TypeID CHILD_MSG     = 12;
    const TypeID FOLDED_INT    = 13;
    const TypeID BUFFER        = 14;
#ifdef MD_LIB_WIN
    // OPTI_PRICE was previously 15, however it has been changed to 2 (INT) to
    // match the behavior of Marquee Server.
    const TypeID OPTI_PRICE    = 2;
#else 
    const TypeID OPTI_PRICE    = 15;
#endif 
    const TypeID UINT          = 16;
    const TypeID LONG_INT      = 17;
    const TypeID ULONG_INT     = 18;
    const TypeID INT64         = 19;
    const TypeID DATETIME      = 20;
    const TypeID PRICE         = 21;
    const TypeID EPOCH_NSEC    = 22;
    
    const TypeID NUM_TYPES     = 23;

    const unsigned char length[] = {
      0,     //RAW  
      1,     //CHAR 
      4,     //INT  
      2,     //SHORT
      4,     //FLOAT
      8,     //DOUBLE
      0,     //STRING
      0,     //VECTOR
      0,     //MSG_T 
      4,     //DATE  
      4,     //TIME  
      12,    //FRACTION
      0,     //CHILD_MSG
      0,     //FOLDED_INT 
      0,     //BUFFER     
      4,     //OPTI_PRICE 
      4,     //UINT       
      4,     //LONG_INT   
      4,     //ULONG_INT  
      8,     //INT64
      8,     //DATETIME
      0,     //PRICE
      8,     //EPOCH_NSEC
    };
  }
}

#endif // __MD_LIB_TYPES_H__
