//forward declare stl container types
namespace std {
    template<class F, class S>                   struct pair;
    template<class T, class A>                   class  vector;
    template<class T, class A>                   class  deque;
    template<class T, class A>                   class  list;
    template<class T, class A>                   class  vector;
    template<class T, class C, class A>          class  set;
    template<class T, class C, class A>          class  multiset;
    template<class K, class V, class C, class A> class  map;
    template<class K, class V, class C, class A> class  multimap;
}


namespace MD{
    
    template<class CharT,class T1, class T2>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::pair<T1,T2> &val);

    template<class CharT,class T,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::vector<T,A> &val);
    
    template<class CharT,class T,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::list<T,A> &val);
    
    template<class CharT,class T,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::deque<T,A> &val);

    template<class CharT, class T,class C,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::set<T,C,A> &val);

    template<class CharT,class T,class C,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::multiset<T,C,A> &val);

    template<class CharT,class K,class V,class C,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::map<K,V,C,A> &val);

    template<class CharT,class K,class V,class C,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::multimap<K,V,C,A> &val);


    class MD_Message;
    extern std::ostream &operator<<(std::ostream & os, const MD::MD_Message & msg);
    //function to escape escape characters
    template<class CharT, class ContainerType>
    inline std::basic_ostream<CharT>&
    to_escaped_string(std::basic_ostream<CharT> &os, const ContainerType &val)
    {
        return os << val;
    }

    template<class CharT, class T1, class T2>
    inline std::basic_ostream<CharT>&
    to_escaped_string(std::basic_ostream<CharT> &os, const std::pair<T1, T2> &val)
    {
       return to_escaped_string(to_escaped_string(os << "(", val.first) << "|", val.second) << ")";
    }

    //common utility function to print contents of std container types
    template<class CharT, class ContainerType>
    inline std::basic_ostream<CharT>&
    print_container(std::basic_ostream<CharT> &os, const ContainerType &val)
    {
        typename ContainerType::const_iterator it(val.begin()), end(val.end());
        if (it == end)
            return os << "[  ]";
        to_escaped_string(os << "[ {", *it) << "}";
        while ((++it) != end)
            to_escaped_string(os << ", {", *it) << "}";
        return os << " ]";
    }

    //operator<< overload for standard containers in MD namespace
    template<class CharT,class T1, class T2>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::pair<T1,T2> &val){
        return to_escaped_string(to_escaped_string(os << "(", val.first) << "|", val.second) << ")"; 
    }

    template<class CharT,class T,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::vector<T,A> &val){
        return print_container(os, val); 
    }

    template<class CharT,class T,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::deque<T,A> &val){
        return print_container(os, val); 
    }

    template<class CharT,class T,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::list<T,A> &val){
        return print_container(os, val); 
    }

    template<class CharT, class T,class C,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::set<T,C,A> &val){
        return print_container(os, val); 
    }

    template<class CharT,class T,class C,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::multiset<T,C,A> &val){
        return print_container(os, val); 
    }

    template<class CharT,class K,class V,class C,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::map<K,V,C,A> &val){
        return print_container(os, val); 
    }

    template<class CharT,class K,class V,class C,class A>
    inline std::basic_ostream<CharT>&
    operator<<(std::basic_ostream<CharT> &os, const std::multimap<K,V,C,A> &val){
        return print_container(os, val); 
    }
}
